@@ -59,7 +59,7 @@ class LockBase(ABC):
 
 
 class LockNoop(LockBase):
-    def __init__(self, *args, **kwargs):  # pylint: disable=super-init-not-called
+    def __init__(self, *args, **kwargs):
         self._lock = False
 
     def lock(self):
@@ -149,9 +149,7 @@ class HardlinkLock(flufl.lock.Lock, LockBase):
         tmp_dir (str): a directory to store claim files.
     """
 
-    def __init__(
-        self, lockfile, tmp_dir=None, **kwargs
-    ):  # pylint: disable=super-init-not-called
+    def __init__(self, lockfile, tmp_dir=None, **kwargs):
         import socket
 
         self._tmp_dir = tmp_dir
@@ -176,7 +174,7 @@ class HardlinkLock(flufl.lock.Lock, LockBase):
         self._owned = True
         self._retry_errnos = []
 
-    def lock(self):  # pylint: disable=arguments-differ
+    def lock(self):
         try:
             super().lock(timedelta(seconds=DEFAULT_TIMEOUT))
         except flufl.lock.TimeOutError:
@@ -187,7 +185,9 @@ class HardlinkLock(flufl.lock.Lock, LockBase):
 
         if self._tmp_dir is not None:
             # Under Windows file path length is limited so we hash it
-            hasher = hashlib.md5(self._claimfile.encode())  # noqa: S324
+            hasher = hashlib.md5(  # noqa: S324
+                self._claimfile.encode()
+            )
             filename = hasher.hexdigest()
             self._claimfile = os.path.join(self._tmp_dir, filename + ".lock")
 
