@@ -3,68 +3,58 @@
 import logging
 import logging.config
 import logging.handlers
+import os
 import sys
 
 import colorama
 
 from dvc.progress import Tqdm
 
-FOOTER = (
-    "\n{yellow}Having any troubles?{nc}"
-    " Hit us up at {blue}https://dvc.org/support{nc},"
-    " we are always happy to help!"
-).format(
-    blue=colorama.Fore.BLUE,
-    nc=colorama.Fore.RESET,
-    yellow=colorama.Fore.YELLOW,
-)
 
-
-def addLoggingLevel(levelName, levelNum, methodName=None):
+def add_logging_level(level_name, level_num, method_name=None):
     """
     Adds a new logging level to the `logging` module and the
     currently configured logging class.
 
-    Uses the existing numeric levelNum if already defined.
+    Uses the existing numeric level_num if already defined.
 
     Based on https://stackoverflow.com/questions/2183233
     """
-    if methodName is None:
-        methodName = levelName.lower()
+    if method_name is None:
+        method_name = level_name.lower()
 
     # If the level name is already defined as a top-level `logging`
     # constant, then adopt the existing numeric level.
-    if hasattr(logging, levelName):
-        existingLevelNum = getattr(logging, levelName)
-        assert isinstance(existingLevelNum, int)
-        levelNum = existingLevelNum
+    if hasattr(logging, level_name):
+        existing_level_num = getattr(logging, level_name)
+        assert isinstance(existing_level_num, int)
+        level_num = existing_level_num
 
-    def logForLevel(self, message, *args, **kwargs):
-        if self.isEnabledFor(levelNum):
-            # pylint: disable=protected-access
-            self._log(levelNum, message, args, **kwargs)
+    def log_for_level(self, message, *args, **kwargs):
+        if self.isEnabledFor(level_num):
+            self._log(level_num, message, args, **kwargs)
 
-    def logToRoot(message, *args, **kwargs):
-        logging.log(levelNum, message, *args, **kwargs)
+    def log_to_root(message, *args, **kwargs):
+        logging.log(level_num, message, *args, **kwargs)
 
     # getLevelName resolves the numeric log level if already defined,
     # otherwise returns a string
-    if not isinstance(logging.getLevelName(levelName), int):
-        logging.addLevelName(levelNum, levelName)
+    if not isinstance(logging.getLevelName(level_name), int):
+        logging.addLevelName(level_num, level_name)
 
-    if not hasattr(logging, levelName):
-        setattr(logging, levelName, levelNum)
+    if not hasattr(logging, level_name):
+        setattr(logging, level_name, level_num)
 
-    if not hasattr(logging.getLoggerClass(), methodName):
-        setattr(logging.getLoggerClass(), methodName, logForLevel)
+    if not hasattr(logging.getLoggerClass(), method_name):
+        setattr(logging.getLoggerClass(), method_name, log_for_level)
 
-    if not hasattr(logging, methodName):
-        setattr(logging, methodName, logToRoot)
+    if not hasattr(logging, method_name):
+        setattr(logging, method_name, log_to_root)
 
 
 class LoggingException(Exception):
     def __init__(self, record):
-        msg = f"failed to log {str(record)}"
+        msg = f"failed to log {record!s}"
         super().__init__(msg)
 
 
@@ -96,9 +86,10 @@ class ColorFormatter(logging.Formatter):
         "CRITICAL": colorama.Fore.RED,
     }
 
-    def __init__(self, log_colors: bool = True) -> None:
+    def __init__(self, log_colors: bool = True, show_traceback: bool = False) -> None:
         super().__init__()
         self.log_colors = log_colors
+        self.show_traceback = show_traceback
 
     def format(self, record) -> str:  # noqa: A003, C901
         record.message = record.getMessage()
@@ -116,8 +107,10 @@ class ColorFormatter(logging.Formatter):
             msg = msg + sep + cause
 
         asctime = ""
-        if _is_verbose():
+        verbose = _is_verbose()
+        if verbose:
             asctime = self.formatTime(record, self.datefmt)
+        if verbose or self.show_traceback:
             if ei and not record.exc_text:
                 record.exc_text = self.formatException(ei)
             if record.exc_text:
@@ -139,7 +132,7 @@ class ColorFormatter(logging.Formatter):
 
 
 class LoggerHandler(logging.StreamHandler):
-    def handleError(self, record):
+    def handleError(self, record):  # noqa: N802
         super().handleError(record)
         raise LoggingException(record)
 
@@ -156,15 +149,15 @@ class LoggerHandler(logging.StreamHandler):
                         self.emit_pretty_exception(exc, verbose=_is_verbose())
                         if not _is_verbose():
                             return
-                    except Exception:  # noqa, pylint: disable=broad-except
-                        pass  # noqa: S110
+                    except Exception:  # noqa: BLE001, S110
+                        pass
 
             msg = self.format(record)
             Tqdm.write(msg, file=self.stream, end=getattr(self, "terminator", "\n"))
             self.flush()
         except (BrokenPipeError, RecursionError):
             raise
-        except Exception:  # noqa, pylint: disable=broad-except
+        except Exception:  # noqa: BLE001
             self.handleError(record)
 
 
@@ -188,7 +181,10 @@ def set_loggers_level(level: int = logging.INFO) -> None:
 def setup(level: int = logging.INFO, log_colors: bool = True) -> None:
     colorama.init()
 
-    formatter = ColorFormatter(log_colors=log_colors and sys.stdout.isatty())
+    color_out = log_colors and bool(sys.stdout) and sys.stdout.isatty()
+    color_err = log_colors and bool(sys.stderr) and sys.stderr.isatty()
+
+    formatter = ColorFormatter(log_colors=color_out)
 
     console_info = LoggerHandler(sys.stdout)
     console_info.setLevel(logging.INFO)
@@ -200,14 +196,15 @@ def setup(level: int = logging.INFO, log_colors: bool = True) -> None:
     console_debug.setFormatter(formatter)
     console_debug.addFilter(exclude_filter(logging.INFO))
 
-    addLoggingLevel("TRACE", logging.DEBUG - 5)
+    add_logging_level("TRACE", logging.DEBUG - 5)
 
     console_trace = LoggerHandler(sys.stdout)
     console_trace.setLevel(logging.TRACE)  # type: ignore[attr-defined]
     console_trace.setFormatter(formatter)
     console_trace.addFilter(exclude_filter(logging.DEBUG))
 
-    err_formatter = ColorFormatter(log_colors=log_colors and sys.stderr.isatty())
+    show_traceback = bool(os.environ.get("DVC_SHOW_TRACEBACK"))
+    err_formatter = ColorFormatter(log_colors=color_err, show_traceback=show_traceback)
     console_errors = LoggerHandler(sys.stderr)
     console_errors.setLevel(logging.WARNING)
     console_errors.setFormatter(err_formatter)
