@@ -1,13 +1,11 @@
-import logging
-from itertools import compress
-
-from funcy.py3 import cat
+from itertools import chain, compress
 
 from dvc.exceptions import InvalidArgumentError
+from dvc.log import logger
 
 from . import locked
 
-logger = logging.getLogger(__name__)
+logger = logger.getChild(__name__)
 
 
 def _joint_status(pairs):
@@ -29,7 +27,7 @@ def _joint_status(pairs):
 
 def _local_status(self, targets=None, with_deps=False, recursive=False):
     targets = targets or [None]
-    pairs = cat(
+    pairs = chain.from_iterable(
         self.stage.collect_granular(t, with_deps=with_deps, recursive=recursive)
         for t in targets
     )
