@@ -1,5 +1,4 @@
 import errno
-import logging
 import os
 import shutil
 import stat
@@ -7,11 +6,12 @@ import sys
 from typing import TYPE_CHECKING
 
 from dvc.exceptions import DvcException
+from dvc.log import logger
 
 if TYPE_CHECKING:
     from dvc.types import StrPath
 
-logger = logging.getLogger(__name__)
+logger = logger.getChild(__name__)
 
 
 class BasePathNotInCheckedPathException(DvcException):
@@ -38,7 +38,7 @@ def contains_symlink_up_to(path: "StrPath", base_path: "StrPath"):
     return contains_symlink_up_to(os.path.dirname(path), base_path)
 
 
-def _chmod(func, p, excinfo):  # noqa: ARG001, pylint: disable=unused-argument
+def _chmod(func, p, excinfo):  # noqa: ARG001
     perm = os.lstat(p).st_mode
     perm |= stat.S_IWRITE
 
