@@ -4,21 +4,29 @@ import os
 import stat
 from textwrap import dedent
 
-import dulwich.errors
-import dulwich.file
 import pytest
+from configobj import ConfigObj
 from funcy import first
 
 from dvc.dvcfile import PROJECT_FILE
-from dvc.exceptions import ReproductionError
+from dvc.env import (
+    DVC_EXP_BASELINE_REV,
+    DVC_EXP_NAME,
+    DVC_ROOT,
+    DVC_STUDIO_OFFLINE,
+    DVC_STUDIO_REPO_URL,
+    DVC_STUDIO_TOKEN,
+    DVC_STUDIO_URL,
+)
+from dvc.exceptions import DvcException, ReproductionError
 from dvc.repo.experiments.exceptions import ExperimentExistsError
 from dvc.repo.experiments.queue.base import BaseStashQueue
 from dvc.repo.experiments.refs import CELERY_STASH
 from dvc.repo.experiments.utils import exp_refs_by_rev
-from dvc.scm import resolve_rev
+from dvc.scm import SCMError, resolve_rev
 from dvc.stage.exceptions import StageFileDoesNotExistError
+from dvc.testing.scripts import COPY_SCRIPT
 from dvc.utils.serialize import PythonFileCorruptedError
-from tests.scripts import COPY_SCRIPT
 
 
 @pytest.mark.parametrize("name", [None, "foo"])
@@ -108,111 +116,6 @@ def test_failed_exp_workspace(
     )
 
 
-def test_apply(tmp_dir, scm, dvc, exp_stage):
-    from dvc.exceptions import InvalidArgumentError
-    from dvc.repo.experiments.exceptions import ApplyConflictError
-
-    results = dvc.experiments.run(exp_stage.addressing, params=["foo=2"], tmp_dir=True)
-    exp_a = first(results)
-
-    results = dvc.experiments.run(
-        exp_stage.addressing, params=["foo=3"], tmp_dir=True, name="foo"
-    )
-    exp_b = first(results)
-
-    with pytest.raises(InvalidArgumentError):
-        dvc.experiments.apply("bar")
-
-    dvc.experiments.apply(exp_a)
-    assert (tmp_dir / "params.yaml").read_text().strip() == "foo: 2"
-    assert (tmp_dir / "metrics.yaml").read_text().strip() == "foo: 2"
-
-    with pytest.raises(ApplyConflictError):
-        dvc.experiments.apply(exp_b, force=False)
-        # failed apply should revert everything to prior state
-    assert (tmp_dir / "params.yaml").read_text().strip() == "foo: 2"
-    assert (tmp_dir / "metrics.yaml").read_text().strip() == "foo: 2"
-
-    dvc.experiments.apply("foo")
-    assert (tmp_dir / "params.yaml").read_text().strip() == "foo: 3"
-    assert (tmp_dir / "metrics.yaml").read_text().strip() == "foo: 3"
-
-
-@pytest.mark.xfail(
-    raises=(
-        dulwich.errors.CommitError,
-        dulwich.file.FileLocked,
-    ),
-    strict=False,
-    reason="See https://github.com/iterative/dvc/issues/8570",
-)
-def test_apply_failed(tmp_dir, scm, dvc, failed_exp_stage, test_queue):
-    from dvc.exceptions import InvalidArgumentError
-
-    dvc.experiments.run(failed_exp_stage.addressing, params=["foo=2"], queue=True)
-    exp_rev = dvc.experiments.scm.resolve_rev(f"{CELERY_STASH}@{{0}}")
-
-    dvc.experiments.run(
-        failed_exp_stage.addressing, params=["foo=3"], queue=True, name="foo"
-    )
-    dvc.experiments.run(run_all=True)
-
-    with pytest.raises(InvalidArgumentError):
-        dvc.experiments.apply("bar")
-
-    dvc.experiments.apply(exp_rev)
-    assert (tmp_dir / "params.yaml").read_text().strip() == "foo: 2"
-
-    dvc.experiments.apply("foo")
-    assert (tmp_dir / "params.yaml").read_text().strip() == "foo: 3"
-
-
-def test_apply_queued(tmp_dir, scm, dvc, exp_stage):
-    from dvc.repo.experiments.exceptions import ApplyConflictError
-
-    metrics_original = (tmp_dir / "metrics.yaml").read_text().strip()
-    dvc.experiments.run(
-        exp_stage.addressing, params=["foo=2"], name="exp-a", queue=True
-    )
-    dvc.experiments.run(
-        exp_stage.addressing, params=["foo=3"], name="exp-b", queue=True
-    )
-    queue_revs = {
-        entry.name: entry.stash_rev
-        for entry in dvc.experiments.celery_queue.iter_queued()
-    }
-
-    dvc.experiments.apply(queue_revs["exp-a"])
-    assert (tmp_dir / "params.yaml").read_text().strip() == "foo: 2"
-    assert (tmp_dir / "metrics.yaml").read_text().strip() == metrics_original
-
-    with pytest.raises(ApplyConflictError):
-        dvc.experiments.apply(queue_revs["exp-b"], force=False)
-
-    dvc.experiments.apply(queue_revs["exp-b"])
-    assert (tmp_dir / "params.yaml").read_text().strip() == "foo: 3"
-    assert (tmp_dir / "metrics.yaml").read_text().strip() == metrics_original
-
-
-def test_apply_untracked(tmp_dir, scm, dvc, exp_stage):
-    from dvc.repo.experiments.exceptions import ApplyConflictError
-
-    results = dvc.experiments.run(exp_stage.addressing, params=["foo=2"])
-    exp = first(results)
-    tmp_dir.gen("untracked", "untracked")
-    tmp_dir.gen("params.yaml", "conflict")
-
-    with pytest.raises(ApplyConflictError):
-        dvc.experiments.apply(exp, force=False)
-
-    assert (tmp_dir / "untracked").read_text() == "untracked"
-    assert (tmp_dir / "params.yaml").read_text() == "conflict"
-
-    dvc.experiments.apply(exp, force=True)
-    assert (tmp_dir / "untracked").read_text() == "untracked"
-    assert (tmp_dir / "params.yaml").read_text().strip() == "foo: 2"
-
-
 def test_get_baseline(tmp_dir, scm, dvc, exp_stage):
     init_rev = scm.get_rev()
     assert dvc.experiments.get_baseline(init_rev) is None
@@ -238,7 +141,7 @@ def test_get_baseline(tmp_dir, scm, dvc, exp_stage):
     assert dvc.experiments.get_baseline(f"{CELERY_STASH}@{{1}}") == init_rev
 
 
-def test_update_py_params(tmp_dir, scm, dvc, test_queue, copy_script):
+def test_update_py_params(tmp_dir, scm, dvc, session_queue, copy_script):
     tmp_dir.gen("params.py", "INT = 1\n")
     stage = dvc.run(
         cmd="python copy.py params.py metrics.py",
@@ -353,11 +256,12 @@ def test_branch(tmp_dir, scm, dvc, exp_stage):
 
     with pytest.raises(InvalidArgumentError):
         dvc.experiments.branch("foo", "branch-exists")
+    dvc.experiments.branch("foo")
     dvc.experiments.branch("foo", "branch-name")
     dvc.experiments.branch(exp_a, "branch-rev")
     dvc.experiments.branch(ref_a, "branch-ref")
 
-    for name in ["branch-name", "branch-rev", "branch-ref"]:
+    for name in ["foo-branch", "branch-name", "branch-rev", "branch-ref"]:
         assert name in scm.list_branches()
         assert scm.resolve_rev(name) == exp_a
 
@@ -435,15 +339,18 @@ def test_packed_args_exists(tmp_dir, scm, dvc, exp_stage, caplog):
         tmp_dir / ".dvc" / "tmp" / BaseExecutor.PACKED_ARGS_FILE,
         "",
         commit="commit args file",
+        force=True,
     )
 
     with caplog.at_level(logging.WARNING):
         dvc.experiments.run(exp_stage.addressing)
         assert "Temporary DVC file" in caplog.text
+    assert not (tmp_dir / ".dvc" / "tmp" / BaseExecutor.PACKED_ARGS_FILE).exists()
 
 
 def test_list(tmp_dir, scm, dvc, exp_stage):
-    baseline_a = scm.get_rev()
+    baseline_old = scm.get_rev()
+
     results = dvc.experiments.run(exp_stage.addressing, params=["foo=2"])
     exp_a = first(results)
     ref_info_a = first(exp_refs_by_rev(scm, exp_a))
@@ -453,28 +360,36 @@ def test_list(tmp_dir, scm, dvc, exp_stage):
     ref_info_b = first(exp_refs_by_rev(scm, exp_b))
 
     tmp_dir.scm_gen("new", "new", commit="new")
+    baseline_new = scm.get_rev()
+
     results = dvc.experiments.run(exp_stage.addressing, params=["foo=4"])
     exp_c = first(results)
     ref_info_c = first(exp_refs_by_rev(scm, exp_c))
 
-    assert dvc.experiments.ls() == {"master": [ref_info_c.name]}
+    assert dvc.experiments.ls() == {baseline_new: [(ref_info_c.name, exp_c)]}
 
     exp_list = dvc.experiments.ls(rev=ref_info_a.baseline_sha)
     assert {key: set(val) for key, val in exp_list.items()} == {
-        baseline_a[:7]: {ref_info_a.name, ref_info_b.name}
+        baseline_old: {(ref_info_a.name, exp_a), (ref_info_b.name, exp_b)}
+    }
+
+    exp_list = dvc.experiments.ls(rev=[baseline_old, baseline_new])
+    assert {key: set(val) for key, val in exp_list.items()} == {
+        baseline_old: {(ref_info_a.name, exp_a), (ref_info_b.name, exp_b)},
+        baseline_new: {(ref_info_c.name, exp_c)},
     }
 
     exp_list = dvc.experiments.ls(all_commits=True)
     assert {key: set(val) for key, val in exp_list.items()} == {
-        baseline_a[:7]: {ref_info_a.name, ref_info_b.name},
-        "master": {ref_info_c.name},
+        baseline_old: {(ref_info_a.name, exp_a), (ref_info_b.name, exp_b)},
+        baseline_new: {(ref_info_c.name, exp_c)},
     }
 
     scm.checkout("branch", True)
     exp_list = dvc.experiments.ls(all_commits=True)
     assert {key: set(val) for key, val in exp_list.items()} == {
-        baseline_a[:7]: {ref_info_a.name, ref_info_b.name},
-        "branch": {ref_info_c.name},
+        baseline_old: {(ref_info_a.name, exp_a), (ref_info_b.name, exp_b)},
+        baseline_new: {(ref_info_c.name, exp_c)},
     }
 
 
@@ -512,11 +427,12 @@ def test_subdir(tmp_dir, scm, dvc, workspace):
     assert resolve_rev(scm, ref_info.name) == exp
 
 
-def test_subrepo(tmp_dir, scm, workspace):
+def test_subrepo(tmp_dir, request, scm, workspace):
     from dvc.testing.tmp_dir import make_subrepo
 
     subrepo = tmp_dir / "dir" / "repo"
     make_subrepo(subrepo, scm)
+    request.addfinalizer(subrepo.dvc.close)
 
     subrepo.gen("copy.py", COPY_SCRIPT)
     subrepo.gen("params.yaml", "foo: 1")
@@ -576,16 +492,6 @@ def test_run_celery(tmp_dir, scm, dvc, exp_stage, mocker):
     assert expected == metrics
 
 
-def test_run_metrics(tmp_dir, scm, dvc, exp_stage, mocker):
-    from dvc.cli import main
-
-    mocker.patch.object(dvc.experiments, "run", return_value={"abc123": "abc123"})
-    show_mock = mocker.patch.object(dvc.metrics, "show", return_value={})
-
-    main(["exp", "run", "-m"])
-    assert show_mock.called_once()
-
-
 def test_checkout_targets_deps(tmp_dir, scm, dvc, exp_stage):
     from dvc.utils.fs import remove
 
@@ -672,7 +578,11 @@ def test_modified_data_dep(tmp_dir, scm, dvc, workspace, params, target, copy_sc
 def test_exp_run_recursive(tmp_dir, scm, dvc, run_copy_metrics):
     tmp_dir.dvc_gen("metric_t.json", '{"foo": 1}')
     run_copy_metrics(
-        "metric_t.json", "metric.json", metrics=["metric.json"], no_exec=True
+        "metric_t.json",
+        "metric.json",
+        metrics=["metric.json"],
+        no_exec=True,
+        name="copy-metric",
     )
     assert dvc.experiments.run(".", recursive=True)
     assert (tmp_dir / "metric.json").parse() == {"foo": 1}
@@ -703,17 +613,42 @@ def test_experiments_workspace_not_log_exception(caplog, dvc, scm):
     assert not caplog.text
 
 
+@pytest.mark.vscode
 def test_run_env(tmp_dir, dvc, scm, mocker):
     dump_run_env = dedent(
         """\
         import os
-        from dvc.env import DVC_EXP_BASELINE_REV, DVC_EXP_NAME
-        with open("DVC_EXP_BASELINE_REV", "w") as f:
-            f.write(os.environ.get(DVC_EXP_BASELINE_REV, ""))
-        with open("DVC_EXP_NAME", "w") as f:
-            f.write(os.environ.get(DVC_EXP_NAME, ""))
+        from dvc.env import (
+            DVC_EXP_BASELINE_REV,
+            DVC_EXP_NAME,
+            DVC_ROOT,
+            DVC_STUDIO_OFFLINE,
+            DVC_STUDIO_REPO_URL,
+            DVC_STUDIO_TOKEN,
+            DVC_STUDIO_URL
+        )
+        for v in (
+            DVC_EXP_BASELINE_REV,
+            DVC_EXP_NAME,
+            DVC_ROOT,
+            DVC_STUDIO_OFFLINE,
+            DVC_STUDIO_REPO_URL,
+            DVC_STUDIO_TOKEN,
+            DVC_STUDIO_URL
+        ):
+            with open(v, "w") as f:
+                f.write(os.environ.get(v, ""))
         """
     )
+    mocker.patch(
+        "dvc.repo.experiments.queue.base.get_studio_config",
+        return_value={
+            "token": "TOKEN",
+            "repo_url": "REPO_URL",
+            "url": "BASE_URL",
+            "offline": "false",
+        },
+    )
     (tmp_dir / "dump_run_env.py").write_text(dump_run_env)
     baseline = scm.get_rev()
     dvc.stage.add(
@@ -721,22 +656,173 @@ def test_run_env(tmp_dir, dvc, scm, mocker):
         name="run_env",
     )
     dvc.experiments.run()
-    assert (tmp_dir / "DVC_EXP_BASELINE_REV").read_text().strip() == baseline
-    assert (tmp_dir / "DVC_EXP_NAME").read_text().strip() != ""
+    assert (tmp_dir / DVC_EXP_BASELINE_REV).read_text().strip() == baseline
+    assert (tmp_dir / DVC_EXP_NAME).read_text().strip()
+    assert (tmp_dir / DVC_ROOT).read_text().strip() == dvc.root_dir
+    assert (tmp_dir / DVC_STUDIO_TOKEN).read_text().strip() == "TOKEN"
+    assert (tmp_dir / DVC_STUDIO_REPO_URL).read_text().strip() == "REPO_URL"
+    assert (tmp_dir / DVC_STUDIO_URL).read_text().strip() == "BASE_URL"
+    assert (tmp_dir / DVC_STUDIO_OFFLINE).read_text().strip() == "false"
 
     dvc.experiments.run(name="foo")
-    assert (tmp_dir / "DVC_EXP_BASELINE_REV").read_text().strip() == baseline
-    assert (tmp_dir / "DVC_EXP_NAME").read_text().strip() == "foo"
+    assert (tmp_dir / DVC_EXP_BASELINE_REV).read_text().strip() == baseline
+    assert (tmp_dir / DVC_EXP_NAME).read_text().strip() == "foo"
 
 
 def test_experiment_unchanged(tmp_dir, scm, dvc, exp_stage):
     dvc.experiments.run(exp_stage.addressing)
     dvc.experiments.run(exp_stage.addressing)
 
-    assert len(dvc.experiments.ls()["master"]) == 2
+    assert len(dvc.experiments.ls()[scm.get_rev()]) == 2
 
 
-def test_experiment_run_dry(tmp_dir, scm, dvc, exp_stage):
+def test_experiment_run_dry(tmp_dir, scm, dvc, exp_stage, mocker):
+    repro = mocker.spy(dvc.experiments, "reproduce_one")
     dvc.experiments.run(exp_stage.addressing, dry=True)
 
     assert len(dvc.experiments.ls()["master"]) == 0
+    assert repro.call_args.kwargs["tmp_dir"] is True
+
+
+def test_clean(tmp_dir, scm, dvc, mocker):
+    clean = mocker.spy(dvc.experiments.celery_queue.celery, "clean")
+    dvc.experiments.clean()
+    clean.assert_called_once_with()
+
+
+def test_experiment_no_commit(tmp_dir):
+    from scmrepo.git import Git
+
+    from dvc.repo import Repo
+
+    Git.init(tmp_dir.fs_path).close()
+
+    repo = Repo.init()
+    assert repo.scm.no_commits
+
+    try:
+        with pytest.raises(SCMError):  # noqa: PT011
+            repo.experiments.ls()
+    finally:
+        repo.close()
+
+
+def test_local_config_is_propagated_to_tmp(tmp_dir, scm, dvc):
+    with dvc.config.edit("local") as conf:
+        conf["cache"]["type"] = "hardlink"
+
+    stage = dvc.stage.add(
+        cmd="cat .dvc/config.local > file", name="foo", outs_no_cache=["file"]
+    )
+    scm.add_commit(["dvc.yaml"], message="add dvc.yaml")
+
+    results = dvc.experiments.run(stage.addressing, tmp_dir=True)
+    exp = first(results)
+    fs = scm.get_fs(exp)
+
+    with fs.open("file") as fobj:
+        conf_obj = ConfigObj(fobj)
+        assert conf_obj["cache"]["type"] == "hardlink"
+
+
+@pytest.mark.parametrize("tmp", [True, False])
+def test_untracked_top_level_files_are_included_in_exp(tmp_dir, scm, dvc, tmp):
+    (tmp_dir / "dvc.yaml").dump(
+        {
+            "metrics": ["metrics.json"],
+            "params": ["params.yaml"],
+            "plots": ["plots.csv"],
+        }
+    )
+    stage = dvc.stage.add(
+        cmd="touch metrics.json && touch params.yaml && touch plots.csv",
+        name="top-level",
+    )
+    scm.add_commit(["dvc.yaml"], message="add dvc.yaml")
+    results = dvc.experiments.run(stage.addressing, tmp_dir=tmp)
+    exp = first(results)
+    fs = scm.get_fs(exp)
+    for file in ["metrics.json", "params.yaml", "plots.csv"]:
+        assert fs.exists(file)
+
+
+@pytest.mark.parametrize("tmp", [True, False])
+def test_copy_paths(tmp_dir, scm, dvc, tmp):
+    stage = dvc.stage.add(
+        cmd="cat file && ls dir",
+        name="foo",
+    )
+    scm.add_commit(["dvc.yaml"], message="add dvc.yaml")
+
+    (tmp_dir / "dir").mkdir()
+    (tmp_dir / "dir" / "file").write_text("dir/file")
+    scm.ignore(tmp_dir / "dir")
+    (tmp_dir / "file").write_text("file")
+    scm.ignore(tmp_dir / "file")
+
+    results = dvc.experiments.run(
+        stage.addressing, tmp_dir=tmp, copy_paths=["dir", "file"]
+    )
+    exp = first(results)
+    fs = scm.get_fs(exp)
+    assert not fs.exists("dir")
+    assert not fs.exists("file")
+
+
+def test_copy_paths_errors(tmp_dir, scm, dvc, mocker):
+    stage = dvc.stage.add(
+        cmd="echo foo",
+        name="foo",
+    )
+    scm.add_commit(["dvc.yaml"], message="add dvc.yaml")
+
+    with pytest.raises(DvcException, match="Unable to copy"):
+        dvc.experiments.run(stage.addressing, tmp_dir=True, copy_paths=["foo"])
+
+    (tmp_dir / "foo").write_text("foo")
+    mocker.patch("shutil.copy", side_effect=OSError)
+
+    with pytest.raises(DvcException, match="Unable to copy"):
+        dvc.experiments.run(stage.addressing, tmp_dir=True, copy_paths=["foo"])
+
+
+def test_mixed_git_dvc_out(tmp_dir, scm, dvc, exp_stage):
+    (tmp_dir / "dir").mkdir()
+    dir_metrics = os.path.join("dir", "metrics.yaml")
+    dvc.stage.add(
+        cmd=f"python copy.py params.yaml {dir_metrics}",
+        metrics=[dir_metrics],
+        params=["foo"],
+        name="copy-file",
+        deps=["copy.py"],
+        force=True,
+    )
+    dvc.stage.add(
+        cmd=f"python copy.py {dir_metrics} metrics.yaml",
+        metrics_no_cache=["metrics.yaml"],
+        name="copy-dir-file",
+        deps=["dir"],
+    )
+    scm.add(["dvc.yaml", "dvc.lock"])
+    scm.commit("add dir stage")
+
+    exp = first(dvc.experiments.run())
+    assert (tmp_dir / "dir" / "metrics.yaml").exists()
+    git_fs = scm.get_fs(exp)
+    assert not git_fs.exists("dir/metrics.yaml")
+
+
+@pytest.mark.parametrize("tmp", [True, False])
+def test_custom_commit_message(tmp_dir, scm, dvc, tmp):
+    stage = dvc.stage.add(
+        cmd="echo foo",
+        name="foo",
+    )
+    scm.add_commit(["dvc.yaml"], message="add dvc.yaml")
+
+    exp = first(
+        dvc.experiments.run(
+            stage.addressing, tmp_dir=tmp, message="custom commit message"
+        )
+    )
+    assert scm.gitpython.repo.commit(exp).message == "custom commit message"
