@@ -1,20 +1,17 @@
 import logging
 import os
-import random
 from datetime import datetime
 from unittest.mock import ANY
 
 import pytest
-from funcy import first, get_in
+from funcy import first
 from scmrepo.exceptions import SCMError
 
 from dvc.cli import main
 from dvc.repo.experiments.executor.base import BaseExecutor, ExecutorInfo, TaskStatus
-from dvc.repo.experiments.queue.base import QueueEntry
-from dvc.repo.experiments.refs import CELERY_STASH, ExpRefInfo
+from dvc.repo.experiments.refs import CELERY_STASH
 from dvc.repo.experiments.utils import EXEC_PID_DIR, EXEC_TMP_DIR, exp_refs_by_rev
 from dvc.utils import relpath
-from dvc.utils.serialize import YAMLFileCorruptedError
 
 LOCK_CONTENTS = {
     "read": {
@@ -40,52 +37,73 @@ def make_executor_info(**kwargs):
     return ExecutorInfo(**kwargs)
 
 
-@pytest.mark.vscode
-def test_show_no_commits(tmp_dir):
-    from dvc.repo import Repo
-    from dvc.scm import Git
+def make_executor(local=None, **kwargs):
+    if local:
+        local_executor = {
+            "root": ANY,
+            "log": ANY,
+            "pid": ANY,
+            "returncode": ANY,
+            "task_id": ANY,
+        }
+        local_executor.update(local)
+    else:
+        local_executor = ANY
+    data = {
+        "state": ANY,
+        "local": local_executor,
+        "name": ANY,
+    }
+    data.update(kwargs)
+    return data
 
-    git = Git.init(tmp_dir.fs_path)
-    assert git.no_commits
 
-    assert Repo.init().experiments.show() == {}
+def make_data(params=None, **kwargs):
+    params = {"data": params or {"foo": 1}}
+    data = {
+        "rev": ANY,
+        "deps": {
+            "copy.py": {
+                "hash": ANY,
+                "size": ANY,
+                "nfiles": None,
+            }
+        },
+        "metrics": {"metrics.yaml": params},
+        "outs": {},
+        "params": {"params.yaml": params},
+        "timestamp": ANY,
+        "meta": ANY,
+    }
+    data.update(kwargs)
+    return data
 
 
 @pytest.mark.vscode
 def test_show_branch_and_tag_name(tmp_dir, scm, dvc, exp_stage):
     with tmp_dir.branch("new/branch", new=True):
         tmp_dir.scm_gen("branch", "branch", "commit")
-        branch_rev = scm.get_rev()
 
     result = dvc.experiments.show(all_branches=True)
-    assert result[branch_rev]["baseline"]["data"]["name"] == "new/branch"
+    expected = [None, "master", "new/branch"]
+    assert [exp.name for exp in result] == expected
 
     scm.tag("new/tag")
     tag_rev = scm.get_rev()
-    result = dvc.experiments.show(all_tags=True)
-    assert result[tag_rev]["baseline"]["data"]["name"] == "new/tag"
+    with scm.detach_head(tag_rev):
+        result = dvc.experiments.show(all_tags=True)
+    expected = [None, "new/tag"]
+    assert [exp.name for exp in result] == expected
 
 
 @pytest.mark.vscode
 def test_show_simple(tmp_dir, scm, dvc, exp_stage):
-    assert dvc.experiments.show()["workspace"] == {
-        "baseline": {
-            "data": {
-                "deps": {
-                    "copy.py": {
-                        "hash": ANY,
-                        "size": ANY,
-                        "nfiles": None,
-                    }
-                },
-                "metrics": {"metrics.yaml": {"data": {"foo": 1}}},
-                "outs": {},
-                "params": {"params.yaml": {"data": {"foo": 1}}},
-                "status": "Success",
-                "executor": None,
-                "timestamp": None,
-            }
-        }
+    assert dvc.experiments.show()[0].dumpd() == {
+        "rev": "workspace",
+        "name": None,
+        "data": make_data(rev="workspace"),
+        "error": None,
+        "experiments": None,
     }
 
 
@@ -97,38 +115,33 @@ def test_show_experiment(tmp_dir, scm, dvc, exp_stage, workspace):
         scm.gitpython.repo.rev_parse(baseline_rev).committed_date
     )
 
-    dvc.experiments.run(exp_stage.addressing, params=["foo=2"], tmp_dir=not workspace)
+    exp_rev = first(
+        dvc.experiments.run(
+            exp_stage.addressing, params=["foo=2"], tmp_dir=not workspace
+        )
+    )
     results = dvc.experiments.show()
-
-    expected_baseline = {
-        "data": {
-            "deps": {
-                "copy.py": {
-                    "hash": ANY,
-                    "size": ANY,
-                    "nfiles": None,
-                }
-            },
-            "metrics": {"metrics.yaml": {"data": {"foo": 1}}},
-            "outs": {},
-            "params": {"params.yaml": {"data": {"foo": 1}}},
-            "status": "Success",
-            "executor": None,
-            "timestamp": timestamp,
-            "name": "master",
-        }
+    assert results[1].dumpd() == {
+        "rev": baseline_rev,
+        "name": "master",
+        "data": make_data(rev=baseline_rev, timestamp=timestamp),
+        "error": None,
+        "experiments": [
+            {
+                "revs": [
+                    {
+                        "rev": exp_rev,
+                        "name": ANY,
+                        "data": make_data(rev=exp_rev, params={"foo": 2}),
+                        "error": None,
+                        "experiments": None,
+                    }
+                ],
+                "executor": None,
+                "name": ANY,
+            }
+        ],
     }
-    expected_params = {"data": {"foo": 2}}
-
-    assert set(results.keys()) == {"workspace", baseline_rev}
-    experiments = results[baseline_rev]
-    assert len(experiments) == 2
-    for rev, exp in experiments.items():
-        if rev == "baseline":
-            assert exp == expected_baseline
-        else:
-            assert exp["data"]["metrics"]["metrics.yaml"] == expected_params
-            assert exp["data"]["params"]["params.yaml"] == expected_params
 
 
 @pytest.mark.vscode
@@ -140,12 +153,28 @@ def test_show_queued(tmp_dir, scm, dvc, exp_stage):
     )
     exp_rev = dvc.experiments.scm.resolve_rev(f"{CELERY_STASH}@{{0}}")
 
-    results = dvc.experiments.show()[baseline_rev]
-    assert len(results) == 2
-    exp = results[exp_rev]["data"]
-    assert exp["name"] == "test_name"
-    assert exp["status"] == "Queued"
-    assert exp["params"]["params.yaml"] == {"data": {"foo": 2}}
+    results = dvc.experiments.show()
+    assert results[1].dumpd() == {
+        "rev": baseline_rev,
+        "name": "master",
+        "data": make_data(rev=baseline_rev),
+        "error": None,
+        "experiments": [
+            {
+                "revs": [
+                    {
+                        "rev": exp_rev,
+                        "name": "test_name",
+                        "data": make_data(rev=exp_rev, params={"foo": 2}, metrics=ANY),
+                        "error": None,
+                        "experiments": None,
+                    }
+                ],
+                "executor": make_executor(state="queued"),
+                "name": "test_name",
+            }
+        ],
+    }
 
     # test that only queued experiments for the current baseline are returned
     tmp_dir.gen("foo", "foo")
@@ -156,143 +185,63 @@ def test_show_queued(tmp_dir, scm, dvc, exp_stage):
     dvc.experiments.run(exp_stage.addressing, params=["foo=3"], queue=True)
     exp_rev = dvc.experiments.scm.resolve_rev(f"{CELERY_STASH}@{{0}}")
 
-    results = dvc.experiments.show()[new_rev]
-    assert len(results) == 2
-    exp = results[exp_rev]["data"]
-    assert exp["status"] == "Queued"
-    assert exp["params"]["params.yaml"] == {"data": {"foo": 3}}
+    results = dvc.experiments.show()
+    assert results[1].dumpd() == {
+        "rev": new_rev,
+        "name": "master",
+        "data": make_data(rev=new_rev),
+        "error": None,
+        "experiments": [
+            {
+                "revs": [
+                    {
+                        "rev": exp_rev,
+                        "name": ANY,
+                        "data": make_data(rev=exp_rev, params={"foo": 3}, metrics=ANY),
+                        "error": None,
+                        "experiments": None,
+                    }
+                ],
+                "executor": make_executor(state="queued"),
+                "name": ANY,
+            }
+        ],
+    }
 
 
 @pytest.mark.vscode
-def test_show_failed_experiment(tmp_dir, scm, dvc, failed_exp_stage):
+def test_show_failed_experiment(tmp_dir, scm, dvc, failed_exp_stage, test_queue):
     baseline_rev = scm.get_rev()
-    timestamp = datetime.fromtimestamp(
-        scm.gitpython.repo.rev_parse(baseline_rev).committed_date
-    )
-
     dvc.experiments.run(failed_exp_stage.addressing, params=["foo=2"], queue=True)
     exp_rev = dvc.experiments.scm.resolve_rev(f"{CELERY_STASH}@{{0}}")
     dvc.experiments.run(run_all=True)
-    experiments = dvc.experiments.show()[baseline_rev]
 
-    expected_baseline = {
-        "data": {
-            "deps": {
-                "copy.py": {
-                    "hash": ANY,
-                    "size": ANY,
-                    "nfiles": None,
-                }
-            },
-            "metrics": {},
-            "outs": {},
-            "params": {"params.yaml": {"data": {"foo": 1}}},
-            "status": "Success",
-            "executor": None,
-            "timestamp": timestamp,
-            "name": "master",
-        }
+    results = dvc.experiments.show()
+    assert results[1].dumpd() == {
+        "rev": baseline_rev,
+        "name": "master",
+        "data": make_data(rev=baseline_rev, metrics=ANY),
+        "error": None,
+        "experiments": [
+            {
+                "revs": [
+                    {
+                        "rev": exp_rev,
+                        "name": ANY,
+                        "data": make_data(rev=exp_rev, params={"foo": 2}, metrics=ANY),
+                        "error": {"msg": "Experiment run failed", "type": ANY},
+                        "experiments": None,
+                    }
+                ],
+                "executor": make_executor(
+                    state="failed",
+                    local={"returncode": 255},
+                ),
+                "name": ANY,
+            }
+        ],
     }
 
-    expected_failed = {
-        "data": {
-            "timestamp": ANY,
-            "params": {"params.yaml": {"data": {"foo": 2}}},
-            "deps": {"copy.py": {"hash": None, "size": None, "nfiles": None}},
-            "outs": {},
-            "status": "Failed",
-            "executor": None,
-            "error": {
-                "msg": "Experiment run failed.",
-                "type": "",
-            },
-            "name": ANY,
-        }
-    }
-
-    assert len(experiments) == 2
-    for rev, exp in experiments.items():
-        if rev == "baseline":
-            assert exp == expected_baseline
-        else:
-            assert rev == exp_rev
-            assert exp == expected_failed
-
-
-@pytest.mark.vscode
-@pytest.mark.parametrize("workspace", [True, False])
-def test_show_checkpoint(tmp_dir, scm, dvc, checkpoint_stage, capsys, workspace):
-    baseline_rev = scm.get_rev()
-    results = dvc.experiments.run(
-        checkpoint_stage.addressing, params=["foo=2"], tmp_dir=not workspace
-    )
-    exp_rev = first(results)
-
-    results = dvc.experiments.show()[baseline_rev]
-    # Assert 4 rows: baseline, 2 checkpoints, and final commit
-    assert len(results) == checkpoint_stage.iterations + 2
-
-    checkpoints = []
-    for rev, exp in results.items():
-        if rev != "baseline":
-            checkpoints.append(rev)
-            assert exp["data"]["checkpoint_tip"] == exp_rev
-
-    capsys.readouterr()
-    assert main(["exp", "show", "--no-pager"]) == 0
-    cap = capsys.readouterr()
-
-    for i, rev in enumerate(checkpoints):
-        if i == 0:
-            name = dvc.experiments.get_exact_name([rev])[rev]
-            name = f"{rev[:7]} [{name}]"
-            fs = "╓"
-        elif i == len(checkpoints) - 1:
-            name = rev[:7]
-            fs = "╨"
-        else:
-            name = rev[:7]
-            fs = "╟"
-        assert f"{fs} {name}" in cap.out
-
-
-@pytest.mark.vscode
-@pytest.mark.parametrize("workspace", [True, False])
-def test_show_checkpoint_branch(tmp_dir, scm, dvc, checkpoint_stage, capsys, workspace):
-    results = dvc.experiments.run(
-        checkpoint_stage.addressing, params=["foo=2"], tmp_dir=not workspace
-    )
-    branch_rev = first(results)
-    if not workspace:
-        dvc.experiments.apply(branch_rev)
-
-    results = dvc.experiments.run(
-        checkpoint_stage.addressing,
-        checkpoint_resume=branch_rev,
-        tmp_dir=not workspace,
-    )
-    checkpoint_a = first(results)
-
-    dvc.experiments.apply(branch_rev)
-    results = dvc.experiments.run(
-        checkpoint_stage.addressing,
-        checkpoint_resume=branch_rev,
-        params=["foo=100"],
-        tmp_dir=not workspace,
-    )
-    checkpoint_b = first(results)
-
-    capsys.readouterr()
-    assert main(["exp", "show", "--no-pager"]) == 0
-    cap = capsys.readouterr()
-
-    for rev in (checkpoint_a, checkpoint_b):
-        ref = dvc.experiments.get_branch_by_rev(rev)
-        ref_info = ExpRefInfo.from_ref(ref)
-        name = f"{rev[:7]} [{ref_info.name}]"
-        assert f"╓ {name}" in cap.out
-    assert f"({branch_rev[:7]})" in cap.out
-
 
 def test_show_filter(
     tmp_dir,
@@ -375,19 +324,21 @@ def test_show_multiple_commits(tmp_dir, scm, dvc, exp_stage):
 
     dvc.experiments.show(num=-2)
 
-    expected = {"workspace", init_rev, next_rev}
+    expected = ["workspace", next_rev, init_rev]
     results = dvc.experiments.show(num=2)
-    assert set(results.keys()) == expected
+    assert [exp.rev for exp in results] == expected
 
-    expected = {"workspace"} | set(scm.branch_revs("master"))
+    expected = ["workspace", *scm.branch_revs("master")]
     results = dvc.experiments.show(all_commits=True)
-    assert set(results.keys()) == expected
+    assert [exp.rev for exp in results] == expected
 
     results = dvc.experiments.show(num=100)
-    assert set(results.keys()) == expected
+    assert [exp.rev for exp in results] == expected
 
 
 def test_show_sort(tmp_dir, scm, dvc, exp_stage, caplog):
+    dvc.experiments.run(exp_stage.addressing, params=["foo=2"])
+
     with caplog.at_level(logging.ERROR):
         assert main(["exp", "show", "--no-pager", "--sort-by=bar"]) != 0
         assert "Unknown sort column" in caplog.text
@@ -401,6 +352,20 @@ def test_show_sort(tmp_dir, scm, dvc, exp_stage, caplog):
     assert main(["exp", "show", "--no-pager", "--sort-by=metrics.yaml:foo"]) == 0
 
 
+def test_show_sort_metric_sep(tmp_dir, scm, dvc, caplog):
+    metrics_path = tmp_dir / "metrics:1.json"
+    metrics_path.write_text('{"my::metric": 1, "other_metric": 0.5}')
+    metrics_path = tmp_dir / "metrics:2.json"
+    metrics_path.write_text('{"my::metric": 2}')
+    dvcyaml_path = tmp_dir / "dvc.yaml"
+    dvcyaml_path.write_text("metrics: ['metrics:1.json', 'metrics:2.json']")
+    dvc.experiments.save()
+    assert (
+        main(["exp", "show", "--no-pager", "--sort-by=metrics:1.json:my::metric"]) == 0
+    )
+    assert main(["exp", "show", "--no-pager", "--sort-by=:other_metric"]) == 0
+
+
 @pytest.mark.vscode
 @pytest.mark.parametrize(
     "status, pid_exists",
@@ -410,14 +375,20 @@ def test_show_sort(tmp_dir, scm, dvc, exp_stage, caplog):
         (TaskStatus.FAILED, False),
     ],
 )
-def test_show_running_workspace(
+def test_show_running(
     tmp_dir, scm, dvc, exp_stage, capsys, caplog, status, pid_exists, mocker
 ):
     from dvc.rwlock import RWLOCK_FILE
+    from dvc_task.proc.process import ProcessInfo
 
+    baseline_rev = scm.get_rev()
     pid_dir = os.path.join(dvc.tmp_dir, EXEC_TMP_DIR, EXEC_PID_DIR)
     lock_file = relpath(os.path.join(dvc.tmp_dir, RWLOCK_FILE), str(tmp_dir))
-    info = make_executor_info(location=BaseExecutor.DEFAULT_LOCATION, status=status)
+    info = make_executor_info(
+        location=BaseExecutor.DEFAULT_LOCATION,
+        status=status,
+        baseline_rev=baseline_rev,
+    )
     pidfile = os.path.join(
         pid_dir,
         "workspace",
@@ -427,188 +398,42 @@ def test_show_running_workspace(
     (tmp_dir / pidfile).dump_json(info.asdict())
     (tmp_dir / lock_file).dump_json(LOCK_CONTENTS)
 
+    mocker.patch.object(ProcessInfo, "load", return_value=mocker.Mock(pid=123))
     mocker.patch("psutil.pid_exists", return_value=pid_exists)
 
-    assert dvc.experiments.show().get("workspace") == {
-        "baseline": {
-            "data": {
-                "deps": {
-                    "copy.py": {
-                        "hash": ANY,
-                        "size": ANY,
-                        "nfiles": None,
-                    }
-                },
-                "metrics": {"metrics.yaml": {"data": {"foo": 1}}},
-                "params": {"params.yaml": {"data": {"foo": 1}}},
-                "outs": {},
-                "status": (
-                    "Running"
-                    if status == TaskStatus.RUNNING and pid_exists
-                    else "Success"
-                ),
-                "executor": (
-                    info.location
-                    if status == TaskStatus.RUNNING and pid_exists
-                    else None
-                ),
-                "timestamp": None,
+    tempdir_active = mocker.spy(dvc.experiments.tempdir_queue, "collect_active_data")
+    celery_active = mocker.spy(dvc.experiments.celery_queue, "collect_active_data")
+    results = dvc.experiments.show()
+    assert results[1].dumpd() == {
+        "rev": ANY,
+        "name": "master",
+        "data": make_data(),
+        "error": None,
+        "experiments": [
+            {
+                "revs": ANY,
+                "executor": make_executor(state="running"),
+                "name": ANY,
             }
-        }
+        ]
+        if pid_exists
+        else None,
     }
-    capsys.readouterr()
-    assert main(["exp", "show", "--csv"]) == 0
-    cap = capsys.readouterr()
-    if status == TaskStatus.RUNNING:
-        if pid_exists:
-            assert "Running" in cap.out
-            assert info.location in cap.out
-        else:
-            cmd = LOCK_CONTENTS["read"]["data/MNIST"][0]["cmd"]
-            pid = LOCK_CONTENTS["read"]["data/MNIST"][0]["pid"]
-            assert (
-                f"Process '{cmd}' with (Pid {pid}), in RWLock-file "
-                f"'{lock_file}' had been killed." in caplog.text
-            )
-            assert f"Delete corrupted RWLock-file '{lock_file}'" in caplog.text
-
-
-def test_show_running_tempdir(tmp_dir, scm, dvc, exp_stage, mocker):
-    baseline_rev = scm.get_rev()
-    run_results = dvc.experiments.run(
-        exp_stage.addressing, params=["foo=2"], tmp_dir=True
-    )
-    exp_rev = first(run_results)
-    exp_ref = first(exp_refs_by_rev(scm, exp_rev))
-
-    queue = dvc.experiments.tempdir_queue
-    stash_rev = "abc123"
-    entries = [
-        QueueEntry(
-            str(tmp_dir / ".dvc" / "tmp" / "foo"),
-            str(tmp_dir / ".dvc" / "tmp" / "foo"),
-            str(exp_ref),
-            stash_rev,
-            exp_ref.baseline_sha,
-            None,
-            exp_ref.name,
-            None,
-        )
-    ]
-    mocker.patch.object(
-        dvc.experiments.tempdir_queue,
-        "iter_active",
-        return_value=entries,
-    )
-    info = make_executor_info(location=BaseExecutor.DEFAULT_LOCATION)
-    pidfile = queue.get_infofile_path(stash_rev)
-    os.makedirs(os.path.dirname(pidfile), exist_ok=True)
-    (tmp_dir / pidfile).dump_json(info.asdict())
-    mock_fetch = mocker.patch.object(
-        dvc.experiments.tempdir_queue,
-        "get_running_exps",
-        return_value={exp_rev: info.asdict()},
-    )
-
-    results = dvc.experiments.show()
-    mock_fetch.assert_has_calls(
-        [mocker.call(True)],
-    )
-    exp_data = get_in(results, [baseline_rev, exp_rev, "data"])
-    assert exp_data["status"] == "Running"
-    assert exp_data["executor"] == info.location
-
-    assert results["workspace"]["baseline"]["data"]["status"] == "Success"
-
-
-def test_show_running_celery(tmp_dir, scm, dvc, exp_stage, mocker):
-    baseline_rev = scm.get_rev()
-    dvc.experiments.run(exp_stage.addressing, params=["foo=2"], queue=True)
-    exp_rev = dvc.experiments.scm.resolve_rev(f"{CELERY_STASH}@{{0}}")
-
-    queue = dvc.experiments.celery_queue
-    entries = list(queue.iter_queued())
-    mocker.patch.object(
-        dvc.experiments.celery_queue,
-        "iter_active",
-        return_value=entries,
-    )
-    info = make_executor_info(location=BaseExecutor.DEFAULT_LOCATION)
-    pidfile = queue.get_infofile_path(entries[0].stash_rev)
-    os.makedirs(os.path.dirname(pidfile), exist_ok=True)
-    (tmp_dir / pidfile).dump_json(info.asdict())
-
-    results = dvc.experiments.show()
-    exp_data = get_in(results, [baseline_rev, exp_rev, "data"])
-    assert exp_data["status"] == "Running"
-    assert exp_data["executor"] == info.location
-
-    assert results["workspace"]["baseline"]["data"]["status"] == "Success"
-
-
-def test_show_running_checkpoint(tmp_dir, scm, dvc, checkpoint_stage, mocker):
-    from dvc.repo.experiments.executor.local import TempDirExecutor
-
-    baseline_rev = scm.get_rev()
-    dvc.experiments.run(checkpoint_stage.addressing, params=["foo=2"], queue=True)
-    queue = dvc.experiments.celery_queue
-    entries = list(queue.iter_queued())
-
-    run_results = dvc.experiments.run(run_all=True)
-    checkpoint_rev = first(run_results)
-    exp_ref = first(exp_refs_by_rev(scm, checkpoint_rev))
-
-    mocker.patch.object(
-        dvc.experiments.celery_queue,
-        "iter_active",
-        return_value=entries,
-    )
-    mocker.patch.object(
-        dvc.experiments.celery_queue,
-        "iter_failed",
-        return_value=[],
-    )
-    pidfile = queue.get_infofile_path(entries[0].stash_rev)
-    info = make_executor_info(
-        git_url="foo.git",
-        baseline_rev=baseline_rev,
-        location=TempDirExecutor.DEFAULT_LOCATION,
-        status=TaskStatus.RUNNING,
-    )
-    os.makedirs(os.path.dirname(pidfile), exist_ok=True)
-    (tmp_dir / pidfile).dump_json(info.asdict())
-
-    mocker.patch.object(BaseExecutor, "fetch_exps", return_value=[str(exp_ref)])
-
-    results = dvc.experiments.show()
-
-    checkpoint_res = get_in(results, [baseline_rev, checkpoint_rev, "data"])
-    assert checkpoint_res["status"] == "Running"
-    assert checkpoint_res["executor"] == info.location
-
-    assert results["workspace"]["baseline"]["data"]["status"] == "Success"
+    tempdir_active.assert_called_once()
+    celery_active.assert_called_once()
 
 
 def test_show_with_broken_repo(tmp_dir, scm, dvc, exp_stage, caplog):
-    baseline_rev = scm.get_rev()
-    exp1 = dvc.experiments.run(exp_stage.addressing, params=["foo=2"])
-    exp2 = dvc.experiments.run(exp_stage.addressing, params=["foo=3"])
-
+    dvc.experiments.run(exp_stage.addressing, params=["foo=2"])
     with open("dvc.yaml", "a", encoding="utf-8") as fd:
         fd.write("breaking the yaml!")
 
-    result = dvc.experiments.show()
-    rev1 = first(exp1)
-    rev2 = first(exp2)
+    results = dvc.experiments.show()
+    assert results[0].error
+    assert results[0].error.type == "YAMLSyntaxError"
 
-    baseline = result[baseline_rev]
-
-    paths = ["data", "params", "params.yaml"]
-    assert get_in(baseline[rev1], paths) == {"data": {"foo": 2}}
-    assert get_in(baseline[rev2], paths) == {"data": {"foo": 3}}
-
-    paths = ["workspace", "baseline", "error"]
-    assert isinstance(get_in(result, paths), YAMLFileCorruptedError)
+    for exp_range in results[1].experiments:
+        assert not any(exp.error for exp in exp_range)
 
 
 def test_show_csv(tmp_dir, scm, dvc, exp_stage, capsys):
@@ -641,8 +466,8 @@ def test_show_csv(tmp_dir, scm, dvc, exp_stage, capsys):
     assert "metrics.yaml:foo,params.yaml:foo,copy.py" in cap.out
     assert f",workspace,baseline,,,3,3,{data_hash}" in cap.out
     assert (
-        "master,{},baseline,{},,1,1,{}".format(
-            baseline_rev[:7], _get_rev_isotimestamp(baseline_rev), data_hash
+        ",master,baseline,{},,1,1,{}".format(  # noqa: UP032
+            _get_rev_isotimestamp(baseline_rev), data_hash
         )
         in cap.out
     )
@@ -706,85 +531,6 @@ def test_show_only_changed(tmp_dir, dvc, scm, capsys, copy_script):
     assert "metrics.yaml:goobar" in cap.out
 
 
-def test_show_parallel_coordinates(tmp_dir, dvc, scm, mocker, capsys, copy_script):
-    from dvc.commands.experiments import show
-
-    webbroser_open = mocker.patch("webbrowser.open")
-    show_experiments = mocker.spy(show, "show_experiments")
-
-    params_file = tmp_dir / "params.yaml"
-    params_data = {
-        "foo": 1,
-        "bar": 1,
-    }
-    (tmp_dir / params_file).dump(params_data)
-
-    dvc.run(
-        cmd="python copy.py params.yaml metrics.yaml",
-        metrics_no_cache=["metrics.yaml"],
-        params=["foo", "bar"],
-        name="copy-file",
-        deps=["copy.py"],
-    )
-    scm.add(
-        [
-            "dvc.yaml",
-            "dvc.lock",
-            "copy.py",
-            "params.yaml",
-            "metrics.yaml",
-            ".gitignore",
-        ]
-    )
-    scm.commit("init")
-
-    dvc.experiments.run(params=["foo=2"])
-
-    assert main(["exp", "show", "--pcp"]) == 0
-    kwargs = show_experiments.call_args[1]
-
-    html_text = (tmp_dir / "dvc_plots" / "index.html").read_text()
-    assert all(rev in html_text for rev in ["workspace", "master"])
-    assert "[exp-" not in html_text
-
-    assert '{"label": "metrics.yaml:foo", "values": [2.0, 1.0]}' in html_text
-    assert '{"label": "params.yaml:foo", "values": [2.0, 1.0]}' in html_text
-    assert '"line": {"color": [1, 0]' in html_text
-    assert '"label": "metrics.yaml:bar"' not in html_text
-    assert '"label": "Created"' not in html_text
-
-    assert main(["exp", "show", "--pcp", "--sort-by", "metrics.yaml:foo"]) == 0
-    kwargs = show_experiments.call_args[1]
-
-    html_text = (tmp_dir / "dvc_plots" / "index.html").read_text()
-    assert '"line": {"color": [2.0, 1.0]' in html_text
-
-    assert main(["exp", "show", "--pcp", "--out", "experiments"]) == 0
-    kwargs = show_experiments.call_args[1]
-
-    assert kwargs["out"] == "experiments"
-    assert (tmp_dir / "experiments" / "index.html").exists()
-
-    assert main(["exp", "show", "--pcp", "--open"]) == 0
-
-    webbroser_open.assert_called()
-
-    params_data = {"foo": 1, "bar": 1, "foobar": 2}
-    (tmp_dir / params_file).dump(params_data)
-    assert main(["exp", "show", "--pcp"]) == 0
-    html_text = (tmp_dir / "dvc_plots" / "index.html").read_text()
-    assert '{"label": "foobar", "values": [2.0, null, null]}' in html_text
-
-    assert main(["exp", "show", "--pcp", "--drop", "foobar"]) == 0
-    html_text = (tmp_dir / "dvc_plots" / "index.html").read_text()
-    assert '"label": "Created"' not in html_text
-    assert '"label": "foobar"' not in html_text
-
-    assert main(["exp", "show", "--pcp", "--drop", "Experiment"]) == 0
-    html_text = (tmp_dir / "dvc_plots" / "index.html").read_text()
-    assert '"label": "Experiment"' not in html_text
-
-
 @pytest.mark.vscode
 def test_show_outs(tmp_dir, dvc, scm, erepo_dir, copy_script):
     params_file = tmp_dir / "params.yaml"
@@ -805,35 +551,52 @@ def test_show_outs(tmp_dir, dvc, scm, erepo_dir, copy_script):
 
     scm.commit("init")
 
-    outs = dvc.experiments.show()["workspace"]["baseline"]["data"]["outs"]
-    assert outs == {
-        "out": {
-            "hash": ANY,
-            "size": ANY,
-            "nfiles": None,
-            "use_cache": True,
-            "is_data_source": False,
-        }
+    results = dvc.experiments.show()
+    assert results[0].dumpd() == {
+        "rev": "workspace",
+        "name": None,
+        "data": make_data(
+            params=ANY,
+            outs={
+                "out": {
+                    "hash": ANY,
+                    "size": ANY,
+                    "nfiles": None,
+                    "use_cache": True,
+                    "is_data_source": False,
+                }
+            },
+        ),
+        "error": None,
+        "experiments": None,
     }
 
     tmp_dir.dvc_gen("out_add", "foo", commit="dvc add output")
-
-    outs = dvc.experiments.show()["workspace"]["baseline"]["data"]["outs"]
-    assert outs == {
-        "out": {
-            "hash": ANY,
-            "size": ANY,
-            "nfiles": None,
-            "use_cache": True,
-            "is_data_source": False,
-        },
-        "out_add": {
-            "hash": ANY,
-            "size": ANY,
-            "nfiles": None,
-            "use_cache": True,
-            "is_data_source": True,
-        },
+    results = dvc.experiments.show()
+    assert results[0].dumpd() == {
+        "rev": "workspace",
+        "name": None,
+        "data": make_data(
+            params=ANY,
+            outs={
+                "out": {
+                    "hash": ANY,
+                    "size": ANY,
+                    "nfiles": None,
+                    "use_cache": True,
+                    "is_data_source": False,
+                },
+                "out_add": {
+                    "hash": ANY,
+                    "size": ANY,
+                    "nfiles": None,
+                    "use_cache": True,
+                    "is_data_source": True,
+                },
+            },
+        ),
+        "error": None,
+        "experiments": None,
     }
 
     with erepo_dir.chdir():
@@ -841,29 +604,38 @@ def test_show_outs(tmp_dir, dvc, scm, erepo_dir, copy_script):
 
     dvc.imp(os.fspath(erepo_dir), "out", "out_imported")
 
-    outs = dvc.experiments.show()["workspace"]["baseline"]["data"]["outs"]
-    assert outs == {
-        "out": {
-            "hash": ANY,
-            "size": ANY,
-            "nfiles": None,
-            "use_cache": True,
-            "is_data_source": False,
-        },
-        "out_add": {
-            "hash": ANY,
-            "size": ANY,
-            "nfiles": None,
-            "use_cache": True,
-            "is_data_source": True,
-        },
-        "out_imported": {
-            "hash": ANY,
-            "size": ANY,
-            "nfiles": None,
-            "use_cache": True,
-            "is_data_source": True,
-        },
+    results = dvc.experiments.show()
+    assert results[0].dumpd() == {
+        "rev": "workspace",
+        "name": None,
+        "data": make_data(
+            params=ANY,
+            outs={
+                "out": {
+                    "hash": ANY,
+                    "size": ANY,
+                    "nfiles": None,
+                    "use_cache": True,
+                    "is_data_source": False,
+                },
+                "out_add": {
+                    "hash": ANY,
+                    "size": ANY,
+                    "nfiles": None,
+                    "use_cache": True,
+                    "is_data_source": True,
+                },
+                "out_imported": {
+                    "hash": ANY,
+                    "size": ANY,
+                    "nfiles": None,
+                    "use_cache": True,
+                    "is_data_source": True,
+                },
+            },
+        ),
+        "error": None,
+        "experiments": None,
     }
 
 
@@ -922,14 +694,9 @@ def test_metrics_renaming(tmp_dir, dvc, scm, capsys, copy_script):
             scm.gitpython.repo.rev_parse(rev).committed_date
         ).isoformat()
 
+    assert f",master,baseline,{_get_rev_isotimestamp(scores_rev)},,1,,1" in cap.out
     assert (
-        "master,{},baseline,{},,1,,1".format(
-            scores_rev[:7], _get_rev_isotimestamp(scores_rev)
-        )
-        in cap.out
-    )
-    assert (
-        ",{},baseline,{},,,1,1".format(
+        ",{},baseline,{},,,1,1".format(  # noqa: UP032
             metrics_rev[:7], _get_rev_isotimestamp(metrics_rev)
         )
         in cap.out
@@ -956,8 +723,6 @@ def test_show_sorted_deps(tmp_dir, dvc, scm, capsys):
 
 @pytest.mark.vscode
 def test_show_queued_error(tmp_dir, scm, dvc, exp_stage, mocker):
-    baseline_rev = scm.get_rev()
-
     dvc.experiments.run(
         exp_stage.addressing, params=["foo=2"], queue=True, name="test_name"
     )
@@ -978,17 +743,16 @@ def test_show_queued_error(tmp_dir, scm, dvc, exp_stage, mocker):
         side_effect=mocker.MagicMock(side_effect=resolve_commit),
     )
 
-    results = dvc.experiments.show()[baseline_rev]
+    results = dvc.experiments.show()[1].experiments
     assert len(results) == 2
-    exp_2 = results[exp_rev_2]["data"]
-    assert exp_2["status"] == "Queued"
-    assert exp_2["params"]["params.yaml"] == {"data": {"foo": 2}}
+    queued = results[0]
+    assert queued.executor.state == "queued"
+    errored = results[1]
+    assert errored.revs[0].error
 
 
 @pytest.mark.vscode
 def test_show_completed_error(tmp_dir, scm, dvc, exp_stage, mocker):
-    baseline_rev = scm.get_rev()
-
     result_2 = dvc.experiments.run(exp_stage.addressing, params=["foo=2"])
     exp_rev_2 = first(result_2)
     commit_2 = scm.resolve_commit(exp_rev_2)
@@ -1005,49 +769,14 @@ def test_show_completed_error(tmp_dir, scm, dvc, exp_stage, mocker):
         "resolve_commit",
         side_effect=mocker.MagicMock(side_effect=resolve_commit),
     )
-    experiments = dvc.experiments.show()[baseline_rev]
-    assert len(experiments) == 2
-    assert exp_rev_2 in experiments
-
-
-@pytest.mark.vscode
-def test_show_checkpoint_error(tmp_dir, scm, dvc, checkpoint_stage, mocker):
-    baseline_rev = scm.get_rev()
-    results = dvc.experiments.run(checkpoint_stage.addressing, params=["foo=2"])
-    exp_rev = first(results)
-    exp_ref = str(first(exp_refs_by_rev(scm, exp_rev)))
-
-    results = dvc.experiments.show()[baseline_rev]
-    # Assert 4 rows: baseline, 2 checkpoints, and final commit
-    assert len(results) == checkpoint_stage.iterations + 2
-
-    checkpoints = {}
-    for rev in results:
-        if rev != "baseline":
-            checkpoints[rev] = scm.resolve_commit(rev)
-    checkpoints[exp_ref] = scm.resolve_commit(exp_ref)
-    checkpoints[baseline_rev] = scm.resolve_commit(baseline_rev)
-
-    failed_rev = random.choice(list(checkpoints.keys()))
-
-    def resolve_commit(rev):
-        if rev == failed_rev:
-            raise SCMError
-        return checkpoints[rev]
-
-    mocker.patch.object(
-        scm,
-        "resolve_commit",
-        side_effect=mocker.MagicMock(side_effect=resolve_commit),
-    )
-    results = dvc.experiments.show()[baseline_rev]
+    results = dvc.experiments.show()[1].experiments
     assert len(results) == 1
+    assert not results[0].revs[0].error
 
 
 @pytest.mark.vscode
 def test_show_baseline_error(tmp_dir, scm, dvc, exp_stage, mocker):
     baseline_rev = scm.get_rev()
-    branch = scm.active_branch()
 
     result_2 = dvc.experiments.run(exp_stage.addressing, params=["foo=2"])
     exp_rev_2 = first(result_2)
@@ -1063,7 +792,7 @@ def test_show_baseline_error(tmp_dir, scm, dvc, exp_stage, mocker):
         "resolve_commit",
         side_effect=mocker.MagicMock(side_effect=resolve_commit),
     )
-    experiments = dvc.experiments.show()[baseline_rev]
-    assert len(experiments) == 1
-    assert experiments["baseline"]["data"] == {"name": branch}
-    assert isinstance(experiments["baseline"]["error"], SCMError)
+
+    results = dvc.experiments.show()
+    assert results[1].error
+    assert len(results[1].experiments) == 1
