@@ -14,6 +14,13 @@ def test_get(mocker):
             "version",
             "--jobs",
             "4",
+            "--config",
+            "myconfig",
+            "--remote",
+            "myremote",
+            "--remote-config",
+            "k1=v1",
+            "k2=v2",
         ]
     )
     assert cli_args.func == CmdGet
@@ -23,7 +30,17 @@ def test_get(mocker):
 
     assert cmd.run() == 0
 
-    m.assert_called_once_with("repo_url", path="src", out="out", rev="version", jobs=4)
+    m.assert_called_once_with(
+        "repo_url",
+        path="src",
+        out="out",
+        rev="version",
+        jobs=4,
+        config="myconfig",
+        force=False,
+        remote="myremote",
+        remote_config={"k1": "v1", "k2": "v2"},
+    )
 
 
 def test_get_url(mocker, capsys):
