@@ -7,18 +7,14 @@ from dvc.commands.imp_url import CmdImportUrl
 from dvc.exceptions import DvcException
 
 
-def test_import_url(mocker):
+def test_import_url(mocker, dvc):
     cli_args = parse_args(
         [
             "import-url",
             "src",
             "out",
-            "--file",
-            "file",
             "--jobs",
             "4",
-            "--desc",
-            "description",
         ]
     )
     assert cli_args.func == CmdImportUrl
@@ -31,21 +27,18 @@ def test_import_url(mocker):
     m.assert_called_once_with(
         "src",
         out="out",
-        fname="file",
         no_exec=False,
         no_download=False,
         remote=None,
         to_remote=False,
-        desc="description",
-        type=None,
-        labels=None,
-        meta=None,
         jobs=4,
+        force=False,
         version_aware=False,
+        fs_config=None,
     )
 
 
-def test_failed_import_url(mocker, caplog):
+def test_failed_import_url(mocker, caplog, dvc):
     cli_args = parse_args(["import-url", "http://somesite.com/file_name"])
     assert cli_args.func == CmdImportUrl
 
@@ -68,17 +61,13 @@ def test_failed_import_url(mocker, caplog):
         ("--no-download", {"no_download": True, "no_exec": False}),
     ],
 )
-def test_import_url_no_exec_download_flags(mocker, flag, expected):
+def test_import_url_no_exec_download_flags(mocker, flag, expected, dvc):
     cli_args = parse_args(
         [
             "import-url",
             flag,
             "src",
             "out",
-            "--file",
-            "file",
-            "--desc",
-            "description",
         ]
     )
 
@@ -90,20 +79,17 @@ def test_import_url_no_exec_download_flags(mocker, flag, expected):
     m.assert_called_once_with(
         "src",
         out="out",
-        fname="file",
         remote=None,
         to_remote=False,
-        desc="description",
         jobs=None,
-        type=None,
-        labels=None,
-        meta=None,
+        force=False,
         version_aware=False,
+        fs_config=None,
         **expected,
     )
 
 
-def test_import_url_to_remote(mocker):
+def test_import_url_to_remote(mocker, dvc):
     cli_args = parse_args(
         [
             "import-url",
@@ -112,8 +98,6 @@ def test_import_url_to_remote(mocker):
             "--to-remote",
             "--remote",
             "remote",
-            "--desc",
-            "description",
         ]
     )
     assert cli_args.func == CmdImportUrl
@@ -126,21 +110,18 @@ def test_import_url_to_remote(mocker):
     m.assert_called_once_with(
         "s3://bucket/foo",
         out="bar",
-        fname=None,
         no_exec=False,
         no_download=False,
         remote="remote",
         to_remote=True,
-        desc="description",
-        type=None,
-        labels=None,
-        meta=None,
         jobs=None,
+        force=False,
         version_aware=False,
+        fs_config=None,
     )
 
 
-@pytest.mark.parametrize("flag", ["--no-exec", "--no-download"])
+@pytest.mark.parametrize("flag", ["--no-exec", "--no-download", "--version-aware"])
 def test_import_url_to_remote_invalid_combination(dvc, mocker, caplog, flag):
     cli_args = parse_args(
         [
@@ -158,7 +139,10 @@ def test_import_url_to_remote_invalid_combination(dvc, mocker, caplog, flag):
     cmd = cli_args.func(cli_args)
     with caplog.at_level(logging.ERROR, logger="dvc"):
         assert cmd.run() == 1
-        expected_msg = "--no-exec/--no-download cannot be combined with --to-remote"
+        expected_msg = (
+            "--no-exec/--no-download/--version-aware cannot be combined with "
+            "--to-remote"
+        )
         assert expected_msg in caplog.text
 
 
