@@ -32,7 +32,7 @@ def test_params_diff(dvc, mocker):
         b_rev="HEAD~1",
         targets=["target"],
         all=True,
-        deps=True,
+        deps_only=True,
     )
     show_diff_mock.assert_not_called()
 
@@ -49,11 +49,11 @@ def test_params_diff_from_cli(dvc, mocker):
 
     params_diff.assert_called_once_with(
         cmd.repo,
-        a_rev=None,
-        b_rev=None,
+        a_rev="HEAD",
+        b_rev="workspace",
         all=False,
         targets=None,
-        deps=False,
+        deps_only=False,
     )
     show_diff_mock.assert_called_once_with(
         {},
@@ -61,15 +61,21 @@ def test_params_diff_from_cli(dvc, mocker):
         markdown=False,
         no_path=False,
         show_changes=False,
-        a_rev=None,
-        b_rev=None,
+        a_rev="HEAD",
+        b_rev="workspace",
     )
 
 
 def test_params_diff_show_json(dvc, mocker, capsys):
     cli_args = parse_args(["params", "diff", "HEAD~10", "HEAD~1", "--json"])
     cmd = cli_args.func(cli_args)
-    mocker.patch("dvc.repo.params.diff.diff", return_value={"params.yaml": {"a": "b"}})
+    mocker.patch(
+        "dvc.repo.params.diff.diff",
+        return_value={
+            "diff": {"params.yaml": {"a": "b"}},
+            "errors": {"rev": Exception},
+        },
+    )
     show_diff_mock = mocker.patch("dvc.compare.show_diff")
 
     assert cmd.run() == 0
