@@ -1,4 +1,5 @@
 import subprocess
+import sys
 
 
 def test_no_remote_imports():
@@ -13,9 +14,9 @@ def test_no_remote_imports():
         "pyarrow",
     }
 
-    code = "import dvc.main, sys; print(' '.join(sys.modules))"
+    code = "import dvc.cli, sys; print(' '.join(sys.modules))"
     res = subprocess.run(
-        ["python", "-c", code], stdout=subprocess.PIPE, check=True
+        [sys.executable, "-c", code], stdout=subprocess.PIPE, check=True
     )
     modules = res.stdout.decode().split()
     assert not set(modules) & remote_modules
